@@ -37,29 +37,38 @@
 (define (get-conf k)
   (hash-ref *conf-hash-table* k))
 
-(define *default-conf-values*
+(define (default-conf-values)
   `(;; for DB namespace
-    ((db dbd) "mysql")
-    ((db port) 3000)
+    ((db enable) false)
+    ((db dbd) mysql)
+    ((db port) 3306)
     ((db addr) "localhost")
     ((db socket) #f)
     ((db username) "root")
     ((db passwd) "")
+    ((db name) ,(or (current-appname) "artanis"))
+    ((db engine) InnoDB)
 
     ;; for server namespace
     ((server info) ,artanis-version)
     ((server nginx) #f)
     ((server charset) "utf-8")
+    ;; FIXME: use local pages
     ((server syspage path) "/etc/artanis/pages")
+    ((server backlog) 128)
     ((server workers) 1)
+    ((server wqlen) 64) ; work queue maxlen
+    ((server trigger) edge)
 
     ;; for host namespace
-    ((host addr) "0.0.0.0")
+    ((host name) #f)
+    ((host addr) "127.0.0.1")
     ((host port) 3000)
+    ((host family) ipv4)
 
     ;; for session namespace
     ((session path) "session")
-    ((session engine) simple)
+    ((session backend) simple)
 
     ;; for upload namespace
     ((upload types) (jpg png gif))
@@ -69,21 +78,24 @@
     ;; ((mail sender) "/usr/sbin/sendmail")
 
     ;; for cache namespace
-    ((cache maxage) 3600)))
+    ((cache maxage) 3600)
+
+    ;; for debug mode
+    ((debug monitor) ""))) ; user specified monitoring paths
 
 ;; Init all fields with default values
-(for-each (lambda (x) (conf-set! (car x) (cadr x))) *default-conf-values*)
+(for-each (lambda (x) (conf-set! (car x) (cadr x))) (default-conf-values))
 
 (define-syntax-rule (->bool x)
   (case (string->symbol (string-downcase x))
     ((true on yes enable) #t)
-    ((fasle off no disable) #f)
+    ((false off no disable) #f)
     (else (error "Invalid boolean item!" x))))
 
 (define-syntax-rule (->list x)
   (map (lambda (e)
          (string->symbol (string-trim-both e)))
-       (string-split x #\,)))
+       (filter (lambda (s) (not (string-null? s))) (string-split x #\,))))
 
 (define-syntax-rule (->none/str x)
   (case (string->symbol (string-downcase x))
@@ -96,15 +108,23 @@
         i
         (error "Invalid integer!" x))))
 
+(define-syntax-rule (->symbol x)
+  (cond
+   ((string? x) (string->symbol x))
+   ((symbol? x) x)
+   (else (error "Invalid value, should be string or symbol!" x))))
+
 (define (parse-namespace-db item)
   (match item
-    (('dbd dbd) (conf-set! '(db dbd) dbd))
+    (('enable usedb) (conf-set! 'use-db? (->bool usedb))) 
+    (('dbd dbd) (conf-set! '(db dbd) (->symbol dbd)))
     (('port port) (conf-set! '(db port) (->integer port)))
     (('addr addr) (conf-set! '(db addr) addr))
     (('name name) (conf-set! '(db name) name))
-    (('socket sock) (conf-set! '(db sock) sock))
+    (('socket sock) (conf-set! '(db sock) (string->symbol sock)))
     (('username username) (conf-set! '(db username) username))
     (('passwd passwd) (conf-set! '(db passwd) passwd))
+    (('engine engine) (conf-set! '(db engine) engine))
     (else (error parse-namespace-db "Config: Invalid item" item))))
 
 (define (parse-namespace-server item)
@@ -114,10 +134,15 @@
     (('charset charset) (conf-set! '(server charset) charset))
     (`(syspage path ,path) (conf-set! '(server syspage path) path))
     (('workers workers) (conf-set! '(server workers) (->integer workers)))
+    (('backlog backlog) (conf-set! '(server backlog) (->integer backlog)))
+    (('wqlen wqlen) (conf-set! '(server wqlen) (->integer wqlen)))
+    (('trigger trigger) (conf-set! '(server trigger) trigger))
     (else (error parse-namespace-server "Config: Invalid item" item))))
 
 (define (parse-namespace-host item)
   (match item
+    (('name name) (conf-set! '(host name) name))
+    (('family family) (conf-set! '(host family) family))
     (('addr addr) (conf-set! '(host addr) addr))
     (('port port) (conf-set! '(host port) (->integer port)))
     (else (error parse-namespace-host "Config: Invalid item" item))))
@@ -125,7 +150,7 @@
 (define (parse-namespace-session item)
   (match item
     (('path path) (conf-set! '(session path) path))
-    (('engine engine) (conf-set! '(session engine) (string->symbol engine)))
+    (('backend backend) (conf-set! '(session backend) (string->symbol backend)))
     (else (error parse-namespace-session "Config: Invalid item" item))))
 
 (define (parse-namespace-upload item)
@@ -144,6 +169,11 @@
     (('maxage maxage) (conf-set! '(cache maxage) (->integer maxage)))
     (else (error parse-namespace-mail "Config: Invalid item" item))))
 
+(define (parse-namespace-debug item)
+  (match item
+    (('monitor monitor) (conf-set! '(debug monitor) (->list monitor)))
+    (else (error parse-namespace-cache "Config: Invalid item" item))))
+
 (define (parse-config-item item)
   (match item
     (('db rest ...) (parse-namespace-db rest))
@@ -153,6 +183,7 @@
     (('upload rest ...) (parse-namespace-upload rest))
     (('mail rest ...) (parse-namespace-mail rest))
     (('cache rest ...) (parse-namespace-cache rest))
+    (('debug rest ...) (parse-namespace-debug rest))
     (((? string-null?)) #t) ; skip
     (else (error parse-config-item "Unsupported config namespace!" item))))
 
@@ -185,7 +216,7 @@
   (define dbd (get-conf '(db dbd)))
   (define port (get-conf '(db port)))
   (define addr (get-conf '(db addr)))
-  (define sock (get-conf '(db sock)))
+  (define sock (and=> (get-conf '(db sock)) (lambda (x) (not (eq? x 'disable)))))
   (define name (get-conf '(db name)))
   (define username (get-conf '(db username)))
   (define passwd (get-conf '(db passwd)))
@@ -194,7 +225,9 @@
    ((or (and port addr sock)
         (and port sock)
         (and addr sock))
-    (error init-inner-database-item "Conf: either addr:port or sock mode!"))
+    (error init-inner-database-item
+           (format #f "Conf: either addr:port or sock mode! port=~a, addr=~a, sock=~a"
+                   port addr sock)))
    ((eq? dbd 'sqlite3) ; sqlite3 needs only dbname
     (conf-set! 'database `(sqlite3 ,username ,passwd ,name)))
    ((and addr port) ; addr:port mode
@@ -208,30 +241,34 @@
 
 (define (init-database-config dbd user passwd dbname)
   ;; if dbd is not specified, it's mysql in default.
-  (conf-set! '(db dbd) (or dbd "mysql"))
+  (conf-set! '(db dbd) (or dbd (get-conf '(db dbd)) 'mysql))
   ;; if username is not specified, it's root in default.
-  (conf-set! '(db username) (or user "root"))
+  (conf-set! '(db username) (or user (get-conf '(db username))"root"))
   ;; if passwd is not specified, it's null in default.
-  (conf-set! '(db passwd) (or passwd ""))
+  (conf-set! '(db passwd) (or passwd (get-conf '(db passwd)) ""))
   ;; if dbname is not specified. it's artanis in default.
-  (conf-set! '(db name) (or dbname "artanis"))
+  (conf-set! '(db name) (or dbname (get-conf '(db name)) "artanis"))
   ;; start to init database
   (init-inner-database-item))
 
 ;; Could be used by cli for specifying user customized config file.
 ;; TODO: Users don't have to call init-config themselves, but call cli:
-;;       art run -c ./my.conf
+;;       art work -c ./my.conf
 ;; And init-server should be called automatically.
 (define current-conf-file (make-parameter #f))
 (define (current-myhost)
   (format #f "http://~a:~a" (get-conf '(host addr)) (get-conf '(host port))))
 
 (define (init-config)
-  (define conf-file (current-conf-file))
+  (define conf-file (current-conf-file)) ; user specified config
   (define fp (open-input-file
               (cond
-               ((and conf-file (file-exists? conf-file)) conf-file)
-               ((file-exists? *default-conf-file*) *default-conf-file*)
+               ((and conf-file (file-exists? conf-file))
+                (format (artanis-current-output) "Loading conf/artanis.conf...")
+                conf-file)
+               ((file-exists? *default-conf-file*)
+                (format (artanis-current-output) "Loading ~a..." *default-conf-file*)
+                *default-conf-file*)
                (else 
                 (error init-config
                        "Fatal error! Do you have /etc/artanis/artanis.conf?")))))
@@ -243,4 +280,5 @@
            (parse-config-item item)
            (lp (read-line fp))))
      (else (error init-config "Invalid line" line))))
-  (init-inner-config-items))
+  (init-inner-config-items)
+  (display "done.\n" (artanis-current-output)))
