@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015
+;;  Copyright (C) 2013,2014,2015,2016
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -18,6 +18,7 @@
 ;;  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (artanis fprm)
+  #:use-module (artanis env)
   #:use-module (artanis utils)
   #:use-module (artanis config)
   #:use-module (artanis route)
@@ -28,11 +29,13 @@
   #:use-module (srfi srfi-11)
   #:use-module (srfi srfi-26)
   #:use-module (ice-9 match)
+  #:use-module (ice-9 format)
   #:export (map-table-from-DB
             make-table-getter
             make-table-setter
             make-table-builder
-            make-table-dropper)
+            make-table-dropper
+            make-table-modifier)
   ;; NOTE:
   ;; We re-export these symbols so that users may use FPRM to handle DB
   ;; independently, without using the web-framework.
@@ -49,6 +52,8 @@
   (format #f "~a ~{~a~^ ~}" n m))
 (define (->1 n m)
   (format #f "~a(~a) ~{~a~^ ~}" n (car m) (cdr m)))
+(define (->0/1 n m)
+  (if (pair? m) (->1 n m) (->0 n m)))
 (define (->2 n m)
   (format #f "~a(~a,~a) ~{~a~^ ~}" n (car m) (cadr m) (cddr m)))
 (define (->n n m)
@@ -80,10 +85,10 @@
 
     ;; TINYINT(size) -128 to 127 normal. 0 to 255 UNSIGNED*.
     ;; The maximum number of digits may be specified in parenthesis
-    ((tinyint) (->1 name args))
+    ((tinyint) (->0/1 name args))
     ;; SMALLINT(size) -32768 to 32767 normal. 0 to 65535 UNSIGNED*.
     ;; The maximum number of digits may be specified in parenthesis
-    ((smallint) (->1 name args))
+    ((smallint) (->0/1 name args))
     ;; MEDIUMINT(size) -8388608 to 8388607 normal. 0 to 16777215 UNSIGNED*.
     ;; The maximum number of digits may be specified in parenthesis
     ((mediumint) (->1 name args))
@@ -133,8 +138,10 @@
 
 (define (->postgresql-type name . args)
   (case name
+    ;; Boolean Type
+    ((boolean) (->0 name args)) ; BOOLEAN Stores TRUE or FALSE values
+
     ;; Numeric Types
-    ((serial) (->0 name args)) ; serial 4 bytes autoincrementing integer 1 to 2147483647
     ((bigserial) (->0 name args)) ;  bigserial 8 bytes large autoincrementing integer 1 to 9223372036854775807
     ((int) (->0 name args))
 
@@ -209,7 +216,6 @@
     ;; CHARACTER VARYING(n) Character string. Variable length. Maximum length n
     ((char-var) (->1 "character varying" args))
     ((binary) (->1 name args)) ; BINARY(n) Binary string. Fixed-length n
-    ((boolean) (->0 name args)) ; BOOLEAN Stores TRUE or FALSE values
     ((varbinary) (->1 name args)) ; VARBINARY(n)
     ;; Binary VARYING(n) Binary string. Variable length. Maximum length n
     ((binary-var) (->1 "binary varying" args))
@@ -238,14 +244,21 @@
     ((array) (->0 name args)) ; ARRAY A set-length and ordered collection of elements
     ((multiset) (->0 name args)) ; MULTISET A variable-length and unordered collection of elements
     ((xml) (->0 name args)) ; XML Stores XML data
+    ((serial) (->0 name args)) ; serial 4 bytes autoincrementing integer 1 to 2147483647
     (else #f)))
 
+(define-syntax-rule (->symbol x)
+  (cond
+   ((string? x) (string->symbol x))
+   ((symbol? x) x)
+   (else (throw 'artanis-err 500 "->symbol: Invalid type!" x))))
+
 ;; (map (lambda (x) (->sql-type (cdr x)))
 ;;      '((name varchar 10) (age int 5) (email varchar 255)))
 ;; ==> ("varchar(10)" "int(5)" "varchar(255)")
 (define-macro (->sql-type name-and-args)
   `(or (apply ->sql-general-type ,name-and-args)
-       (apply ,(symbol-append '-> (string->symbol (get-conf '(db dbd))) '-type)
+       (apply ,(symbol-append '-> (->symbol (get-conf '(db dbd))) '-type)
         ,name-and-args)))
 
 (define (make-table-dropper rc/conn)
@@ -254,13 +267,57 @@
      ((route-context? rc/conn) (rc-conn rc/conn))
      ((<connection>? rc/conn) rc/conn)
      (else (throw 'artanis-err 500 "make-table-dropper: Invalid rc or conn!" rc/conn))))
-  (lambda (name)
-    (DB-query conn (->sql drop table if exists name))))
+  (lambda* (name #:key (dump #f))
+    (let ((sql (->sql drop table if exists name)))
+      (cond
+       ((not dump) (DB-query conn sql))
+       (else sql)))))
+
+(define *exception-opts*
+  '(#:no-edit))
+(define (is-exception-opt? x)
+  (memq x *exception-opts*))
+
+(define (->mysql-opts x opts)
+  (define-syntax-rule (->value x)
+    (format #f "~:@(~a~) ~a" x (kw-arg-ref opts x)))
+  (case x
+    ((#:not-null) "NOT NULL")
+    ((#:null) "NULL")
+    ((#:default) (->value x))
+    ((#:unique) "UNIQUE")
+    ((#:unique-key) "UNIQUE KEY")
+    ((#:primary-key) "PRIMARY KEY")
+    ((#:key) "KEY")
+    ((#:auto-increment) "AUTO_INCREMENT")
+    ((#:comment) (->value x))
+    ((#:column-format) "COLUMN_FORMAT")
+    ((#:storage) (->value x))
+    ((#:reference-definition) "reference_definition")
+    (else
+     (if (is-exception-opt? x)
+         ""
+         (throw 'artanis-err 500 "Invalid opts for MySQL table definition!" x)))))
+
+(define (->postgresql-opts dbd opts)
+  (format #t "PostgreSQL migration hasn't been supported yet!~%")
+  #t)
+(define (->sqlite3-opts dbd opts)
+  (format #t "SQLite3 migration hasn't been supported yet!~%")
+  #t)
+
+(define *table-builder-opts-handler*
+  `((mysql . ,->mysql-opts)
+    (postgresql . ,->postgresql-opts)
+    (sqlite3 . ,->sqlite3-opts)))
+
+(define (get-table-builder-opts-handler)
+  (assoc-ref *table-builder-opts-handler* (get-conf '(db dbd))))
 
 ;; NOTE:
 ;; 1. Use primiary-keys for specifying primary keys, don't specify it in defs directly.
 ;;    Because we're not going to support foreign keys, so we need to record keys in closures for sync.
-;; 2. But, in FPRM, it'd be STATELESS, so closures shouldn't be stated.
+;; 2. But, in FPRM, any state should be IMMUTABLE.
 ;; 3. SOLUTION: use a access-hook for specified table mapping, but there'd be a fine way to avoid
 ;;              write sql directly each time.
 (define* (make-table-builder rc/conn)
@@ -271,19 +328,31 @@
      (else (throw 'artanis-err 500 "make-table-builder: Invalid rc or conn!" rc/conn))))
   (define (table-drop! tname)
     (DB-query conn (->sql drop table if exists tname)))
-  (define (->types x pks)
-    (format #f "~{~a~^ ~}"
-            (list (car x)
-                  (->sql-type
-                   (if (memq (car x) pks)
-                       `(,@(cdr x) primary key)
-                       (cdr x))))))
+  (define (->opts opts)
+    (let ((h (get-table-builder-opts-handler)))
+      (string-join
+       (srfi-1:fold-right
+        (lambda (x p)
+          (let ((o (h x opts)))
+            (cons o p)))
+        '() opts)
+       " ")))
+  (define (->type/opts x)
+    (match x
+      ((types ... (opts ...)) (values types (if (null? opts) "" (->opts opts))))
+      ((types ...) (values types ""))
+      (else (throw 'artanis-err 500 "->type/opts: invalid definition of the table!" x))))
+  (define (->types x)
+    (call-with-values
+        (lambda () (->type/opts x))
+      (lambda (types opts)
+        (format #f "~{~a~^ ~}" (list (car types) (->sql-type (cdr types)) opts)))))
   ;; TODO: We need a mechanism to sync tables constrained by foreign-keys, since some DB doesn't
   ;;       support foreign keys directly, so we have to provide it outside.
   ;; TODO: who to deal with constrained tables without foreign-keys in stateless?
-  (lambda* (tname defs #:key (if-exists? #f) (primary-keys '()) (engine #f))
-    (let* ((types (map (cut ->types <> primary-keys) defs))
-           (aaa (format #t "~a~%" types))
+  (lambda* (tname defs #:key (if-exists? #f) (engine (get-conf '(db engine)))
+                  (dump #f) (primary-keys '()))                  
+    (let* ((types (map ->types defs))
            (sql (case if-exists?
                   ((overwrite drop)
                    (table-drop! tname)
@@ -291,39 +360,50 @@
                   ((ignore)
                    (->sql create table if not exists tname (types) engine))
                   (else (->sql create table tname (types) engine)))))
-      (DB-query conn sql)
-      (lambda cmd
-        (match cmd
-          ('(valid?) (db-conn-success? conn))
-          ('(primary-keys) primary-keys)
-          (`(add-primary-keys ,keys)
-           (DB-query conn (->sql alter table tname add primary key keys)))
-          ('(drop-primary-keys)
-           (DB-query conn (->sql alter table tname drop primary key)))
-          ;; TODO
-          (else (throw 'artanis-err 500 "make-table-builder: Invalid cmd!" cmd)))))))
+      (cond
+       ((not dump)
+        (DB-query conn sql)
+        (lambda cmd
+          (match cmd
+            ('(valid?) (db-conn-success? conn))
+            ('(primary-keys) primary-keys)
+            (`(add-primary-keys ,keys)
+             (DB-query conn (->sql alter table tname add primary key keys)))
+            ('(drop-primary-keys)
+             (DB-query conn (->sql alter table tname drop primary key)))
+            ;; TODO
+            (_ (throw 'artanis-err 500 "make-table-builder: Invalid cmd!" cmd)))))
+       (else sql)))))
 
-;; make-table-setter is actually a mapping from `update' in SQL
+;; make-table-setter could be mapped to UPDATE or INSERT, depends on condition.
 ;; Grammar:
 ;; UPDATE table_name
 ;;        SET column1=value1,column2=value2,...
 ;;        WHERE some_column=some_value;
+;;
+;; === When there's no conditions ===
+;;
+;; INSERT INTO table_name
+;;        (column1, column1 ...) values (value1, value2 ...)
 (define (make-table-setter rc/conn)
   (define (->kvp kargs)
     (let lp((next kargs) (kvs '()) (w ""))
       (match next
-       (((? keyword? k) v rest ...)
-        (lp rest (cons (list (keyword->symbol k) v) kvs) w))
-       (((? string? wcond))
-        (lp (cdr next) kvs wcond))
-       (() (values kvs w))
-       (else (throw 'artanis-err 500 "->kvp: invalid kargs" next)))))
+        (((? keyword? k) v rest ...)
+         (lp rest (cons (list (keyword->symbol k) v) kvs) w))
+        (((? string? wcond))
+         (lp (cdr next) kvs wcond))
+        (() (values kvs w))
+        (else (throw 'artanis-err 500 "->kvp: invalid kargs" next)))))
   (define (->kv kvp) (srfi-1:unzip2 kvp))
   (lambda (tname . kargs)
     (let-values (((kvp wcond) (->kvp kargs)))
       (let ((sql (if (string-null? wcond)
                      (let-values (((k v) (->kv kvp)))
                        (->sql insert into tname k values v))
+                     ;; NOTE: If there's no cond string (say, `where' clauses), you MUSTN'T
+                     ;;       use UPDATE because it'll effect all the records!!! In such case,
+                     ;;       you should use INSERT.
                      (->sql update tname set kvp wcond)))
             (conn (cond 
                    ((route-context? rc/conn) (rc-conn rc/conn))
@@ -334,27 +414,45 @@
 (define (make-table-getter rc/conn)
   (define (->ret ret)
     (match ret
-      ((? integer? n) (format #f "limit ~a " n))
-      ('top "limit 1 ")
+      ((? integer? n) (format #f " limit ~a " n))
+      ('top " limit 1 ")
       ('all "")
-      (else #f)))
+      (_ #f)))
   (define (->group-by group-by)
     (match group-by
       ((? list columns)
-       (format #f "~{~a~^,~} " columns))
-      (else #f)))
+       (format #f " ~{~a~^,~} " columns))
+      (_ #f)))
   (define (->order-by order-by)
     (match order-by
       ((columns ... (? (cut memq <> '(asc desc)) m))
-       (format #f "~{~a~^,~} ~a " columns m))
-      (else #f)))
-  (define (->opts ret group-by order-by)
+       (format #f " ~{~a~^,~} ~a " columns m))
+      (_ #f)))
+  (define (->opts ret group-by order-by cnd foreach)
     (define-syntax-rule (-> x tox)
       (or (and=> x tox) ""))
+    (define-syntax-rule (cond-combine c lst)
+      (cond
+       ((not (string? c))
+        (throw 'artanis-err 500 "Invalid #:condition" c))
+       ((not (list? lst))
+        (throw 'artanis-err 500 "Invalid #:foreach" lst))
+       (else
+        (match lst
+          (() c)
+          ((column (? list? vals))
+           (format #f " ~a ~a in (~{'~a'~^,~}) "
+                   (if (string-null? c) " where " (string-append c " and "))
+                   column vals))
+          (_
+           (throw 'artanis-err 500
+                  "Invalid #:foreach, should be (column (val1 val2 val3 ...))"
+                  lst))))))
     (string-concatenate
-     (list (-> ret ->ret)
-           (-> group-by ->group-by)
-           (-> order-by ->order-by))))
+     (list (-> group-by ->group-by)
+           (-> order-by ->order-by)
+           (cond-combine cnd foreach)
+           (-> ret ->ret))))
   (define (->mix columns functions)
     `(,@columns ,@(map (lambda (f) (format #f "~a(~{~a~^,~})" (car f) (cdr f))) functions)))
   (lambda* (tname #:key (columns '(*)) ; get all (*) in default
@@ -381,15 +479,81 @@
                   ;; SELECT column_name(s)
                   ;;        FROM table_name
                   ;;        ORDER BY column_name [ASC|DESC]
+                  (condition "")
+                  ;; Conditions, for example: #:cond (where #:name "nala")
+                  (foreach '())
+                  ;; #:foreach accepts an associative list according to this form:
+                  ;; (column-name (val1 val2 val3 ...))
+                  ;; for example:
+                  ;; #:foreach '(city ("sz" "bj" "sh"))
+                  ;; NOTE: This is useful to avoid N+1 query problem.
+                  (dump #f)
+                  ;; Dump SQL to a string as result, when #:dump set to #t, it won't do query operation. 
                   )
     (let ((sql (format #f "select ~{~a~^,~} from ~a ~a;"
-                       (->mix columns functions) tname (->opts ret group-by order-by)))
+                       (->mix columns functions) tname (->opts ret group-by order-by condition foreach)))
           (conn (cond
                  ((route-context? rc/conn) (rc-conn rc/conn))
                  ((<connection>? rc/conn) rc/conn)
-                 (else (throw 'artanis-err 500 "make-table-getter: Invalid rc or conn!" rc/conn)))))
-      (DB-query conn sql)
-      (DB-get-all-rows conn))))
+                 (else (throw 'artanis-err 500
+                              "make-table-getter: Invalid rc or conn!" rc/conn)))))
+      (cond
+       ((not dump)
+        (DB-query conn sql)
+        (DB-get-all-rows conn))
+       (else sql)))))
+
+(define (make-table-modifier rc/conn)
+  (define (table-add tname col t)
+    (->sql alter table tname add col t))
+  (define (table-drop tname col)
+    (->sql alter table tname drop column col))
+  (define (table-alter tname col t)
+    (case (get-conf '(db dbd))
+      ((mysql) (->sql alter table tname modify column col t))
+      ((postgresql) (->sql alter table tname alter column col t))
+      ((sqlite3) (throw 'artanis-err 500 "SQLite3 doesn't support table modification!"))
+      (else (throw 'artanis-err 500 "table-alter: Unsupported DBD!" (get-conf '(db dbd))))))
+  (define (table-rename tname new)
+    (case (get-conf '(db dbd))
+      ((mysql) (->sql alter table tname rename new))
+      ((postgresql sqlite3) (->sql alter table tname rename to new))
+      (else (throw 'artanis-err 500 "table-alter: Unsupported DBD!" (get-conf '(db dbd))))))
+  (define (column-drop tname cname)
+    (case (get-conf '(db dbd))
+      ((mysql postgresql) (->sql alter table tname drop column cname))
+      ((sqlite3) (throw 'artanis-err 500 "SQLite3 doesn't support table column modification!"))
+      (else (throw 'artanis-err 500 "table-alter: Unsupported DBD!" (get-conf '(db dbd))))))
+  (define (column-rename tname old new . type)
+    (case (get-conf '(db dbd))
+      ((mysql) (let ((t (car type))) (->sql alter table tname change column old new t)))
+      ((postgresql) (->sql alter table tname rename column old new))
+      ((sqlite3) (throw 'artanis-err 500 "SQLite3 doesn't support table column modification!"))
+      (else (throw 'artanis-err 500 "table-alter: Unsupported DBD!" (get-conf '(db dbd))))))
+  (define (index-rename tname old new)
+    (case (get-conf '(db dbd))
+      ((mysql) (->sql alter table tname index old rename to new))
+      ((postgresql) (->sql alter table tname index old rename to new))
+      ((sqlite3) (throw 'artanis-err 500 "SQLite3 doesn't support table column modification!"))
+      (else (throw 'artanis-err 500 "table-alter: Unsupported DBD!" (get-conf '(db dbd))))))
+  (define (gen-sql tname op args)
+    (case op
+      ((add) (apply table-add tname args))
+      ((drop) (apply table-drop tname args))
+      ((column-drop) (apply column-drop args))
+      ((alter) (apply table-alter tname args))
+      ((rename) (apply table-rename tname args))
+      ((column-rename) (apply column-rename tname args))
+      ((index-rename) (apply index-rename args))
+      (else (throw 'artanis-err 500 "make-table-modifier: Invalid op!" op))))
+  (lambda (tname op . args)
+    (let ((sql (gen-sql tname op args))
+          (conn (cond 
+                 ((route-context? rc/conn) (rc-conn rc/conn))
+                 ((<connection>? rc/conn) rc/conn)
+                 (else (throw 'artanis-err 500
+                              "make-table-modifier: Invalid rc or conn!" rc/conn)))))
+      (DB-query conn sql))))
 
 ;; NOTE: the name of columns is charactar-caseless, at least in MySQL/MariaDB.
 (define (map-table-from-DB rc/conn)
@@ -402,6 +566,7 @@
   (define setter (make-table-setter conn))
   (define builder (make-table-builder conn))
   (define dropper (make-table-dropper conn))
+  (define modifier (make-table-modifier conn))
   ;; NOTE:
   ;; It maybe inefficient to fetch table-schema without any cache, because the request session may generate
   ;; table-schema each time. Although we may build a cache or delayed mechanism here, there's one reason
@@ -413,7 +578,7 @@
   ;; PS: I'm not boasting that the whole Artanis would be stateless, but FPRM should do it as possible.
   ;; I maybe wrong and fail, but it's worth to try.
   (define (get-table-schema tname)      
-    (let* ((sql (->sql select ("lcase(column_name)") from
+    (let* ((sql (->sql select '("lcase(column_name)") from
                        (select * from 'information_schema.columns (having #:table_name tname))
                        as 'tmp_alias))
            (sch (DB-get-all-rows (DB-query conn sql))))
@@ -424,6 +589,9 @@
     (define schema (get-table-schema tname))
     (define-syntax-rule (-> c) (map (cut normalize-column <> ci?) c))
     (and (srfi-1:every (lambda (x) (memq x schema)) (-> args)) #t))
+  (define (table-exists? tname)
+    (DB-query rc/conn (format #f "show tables like '~a';" tname))
+    (DB-get-top-row rc/conn))
   (lambda (cmd tname . args)
     (define-syntax-rule (->call func)
       (apply func (cons tname args)))
@@ -431,6 +599,11 @@
       ((valid?) (db-conn-success? conn))
       ((get) (->call getter))
       ((set) (->call setter))
+      ((table-exists?) (table-exists? tname))
+      ((try-create try-build)
+       (if (table-exists? tname)
+           (format (artanis-current-output) "Table `~a' already exists!~%" tname)
+           (->call builder)))
       ((create build) (->call builder))
       ((remove delete drop) (->call dropper))
       ;; (_ exists? 'Persons 'city 'lastname)
@@ -439,4 +612,18 @@
       ((ci-check ci-exists?) (apply checker #t tname args))
       ;; schema is always in downcase.
       ((schema) (get-table-schema tname))
+      ((mod) (->call modifier))
       (else (throw 'artanis-err 500 "map-table-from-DB: Invalid cmd" cmd)))))
+
+(define* (fprm-find mt tname val #:key (id 'id))
+  (mt 'get tname #:condition (where (symbol->keyword id) val)))
+
+(define *table-mapper-handlers*
+  `((find . ,fprm-find)
+    ))
+
+(define (create-table-mapper tname rc/conn)
+  (let ((mt (map-table-from-DB rc/conn)))
+    (lambda (cmd . args)
+      (and=> (assoc-ref *table-mapper-handlers* cmd)
+             (lambda (h) (apply h mt tname args))))))
